//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.


// Interface to msis00
// NB: a limited number of checks is done in this function. 
//
// Inputs: 
// year: (1xN) - should be an integer value
// doy: day of year (1xN) - should be an integer value
// secs: seconds in day - UTC (1xN)
// glon, glat, galt: geodetic coordinates [m, rad] (1xN)
// lst: local time (1xN)
// f107: flux 10.7 (1xN)
// f107a: mean flux 10.7 (1xN)
// ap: geomagnetic indices (1xN or 7xN)
// nb_ap: 1 or 7 - (1x1) should be an integer value
//        
// Outputs: 
// dens: total density [kg/m^3] (1xN)
// temp: temperature at current position [K] (1xN)
// temp_exo: exospheric temperature [K] (1xN)
// nb_part: density of species [1/m^3] (8xN)
                
function [dens, temp, temp_exo, nb_part] = CLx_msis00(year, doy, secs, glon, glat, galt, lst, f107, f107a, ap, nb_ap)

if (argn(2) <> 11)
  CLx__error("Invalid number of arguments"); 
end

if (nb_ap <> 1 & nb_ap <> 7)
  CLx__error("Invalid value for argument ap"); 
end

N = length(year);
p = %nan * zeros(9,N);
t = %nan * zeros(2,N);

// low level (C) interface expects always the same size for ap
// whatever the number of ap that should be used. 
if (size(ap, 1) == 1)
  ap = ones(7,1) * ap;  // 7 identical rows
end

// call gateway function: 
// convert inputs to degrees and km
// round off integer quantities and convert to int32 type
// for meaning of p and t => see CLx__msis2000
[p, t] = CLx__msis00( ..
      int32(N), ..
      int32(round(year)), ..
      int32(round(doy)), ..
      secs, ..
      glon * 180 / %pi, .. 
      glat * 180 / %pi, ..
      galt / 1000, ..
      lst, ..
      f107, ..
      f107a, ..
      ap, ..
      int32(round(nb_ap)));

dens = p(6,:); 
nb_part = p([1:5, 7:9], :); 
temp_exo = t(1,:); 
temp = t(2,:); 

endfunction

