//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

// Converts the params (Scilab) structure into a "jparams" (Java) object
// params: model parameters for CLx_stela_extrap, CLx_stela_deriv ...

function [jparams] = CLx__createStelaParams(params)

  // Standard parameters
  Params1 = [["mass"; "double"], ["central_enabled"; "boolean"], ["zonal_enabled"; "boolean"], ..
             ["zonal_maxDeg"; "integer"], ["tesseral_enabled"; "boolean"], ["tesseral_maxDeg"; "integer"], ..
             ["tesseral_minPeriod"; "double"], ["thirdBody_enabled"; "boolean"], ["thirdBody_bodies"; "string"], ..
             ["drag_enabled"; "boolean"], ["drag_coefType"; "string"], ..
             ["drag_area"; "double"], ["drag_coef"; "double"], ["drag_solarActivityType"; "string"], ..
             ["drag_solarActivityFlux"; "double"], ["drag_solarActivityAp"; "double"], ..
             ["srp_enabled"; "boolean"], ["srp_area"; "double"], ["srp_coef"; "double"], ..
             ["solidTides_enabled"; "boolean"], ..
             ["appAcc_enabled"; "boolean"], ["appAcc_refFrame"; "string"], ..
             ["integrator_step"; "double"]]; 
  
  // Optional parameters 
  Params2 = [["drag_atmosphericModel"; "string"], ..
             ["srp_nbQuadPoints"; "integer"], ..
             ["srp_eclipses"; "boolean"], ..
             ["drag_nbQuadPoints"; "integer"], ["drag_nbComputeSteps"; "integer"], ..
             ["appAcc_nbQuadPoints"; "integer"], ["thirdBody_degree"; "integer"], ["reentryAltitude"; "double"], ..
             ["drag_cookWallTemp"; "double"], ["drag_cookAccomodCoef"; "double"], ..
             ["solidTides_bodies"; "string"], ["zonal_computationMeth"; "string"]]; 
             
  // More advanced parameters 
  // NB: all may not be not documented (volontarily)
  // WARNING: changing path of physical constants file is risky! 
  Params2 = [ Params2, ..
              [["solarActivityFile"; "string"], ..
               ["aeroCoefFile"; "string"], ..
               ["physicalConstantsFile"; "string"]] ]; 
                       
  AllParams = [Params1, Params2]; 

  // Check type of "params" argument
  if (typeof(params) <> "st")
    CLx__error("Incorrect type for argument params: not a structure");
  end
  
  // Advanced parameters (optional)
  // if parameter exists and value is "" => use standard path (in default location)  
  // NB: not necessary to change value for solar activity or aero coef => value remains ""
  NAMES = ["physicalConstantsFile", "solarActivityFile", "aeroCoefFile"]; 
  FILENAMES = ["stela_physical_parameters", "", ""]; // relative to STELA_ROOT 
  
  // create local copy of params
  params = params; 

  for (k = 1 : size(NAMES,2))
    name = NAMES(k);   
    if (isfield(params, name))
      value = stripblanks(params(name), %t); 
      if (value == "")
        params(name) = FILENAMES(k); 
      end
    end
  end

  // Chage of name (compatibility with previous version)
  NAMES = ["thirdbody_enabled", "thirdbody_bodies", "thirdbody_degree"]; 
  NEWNAMES = ["thirdBody_enabled", "thirdBody_bodies", "thirdBody_degree"];  

  for (k = 1 : size(NAMES,2)) 
    if (isfield(params, NAMES(k)))
      if (~isfield(params, NEWNAMES(k)))
        params(NEWNAMES(k)) = params(NAMES(k)); 
        params(NAMES(k)) = null(); 
      else
        CLx__error("Fieldname defined more than once in params structure: " + NEWNAMES(k)); 
      end
    end
  end
  
  // Compatibility (ref_frame instead of appAcc_refFrame)
  if (isfield(params, "ref_frame"))
    mprintf("*** CelestLabX (warning): Obsolete use of ref_frame keyword in params structure.\n"); 
    mprintf("*** ref_frame used, appAcc_refFrame ignored. \n"); 
    params.appAcc_refFrame = params.ref_frame; 
    params.ref_frame = null(); 
  end
  
  // Process all fields of "params" structure
  names = matrix(fieldnames(params), 1, -1);  
  [nb, Loc] = members(names, AllParams(1,:)); 
  
  
  // Check all names are correct
  I = find(nb <> 1); 
  if (I <> [])
    CLx__error("Incorrect fieldname(s) in params structure: " + strcat(names(I), " "));
  end

  // initialize java structure
  jimport fr.cnes.celestlab.celestlabx.generic.Params;
  jparams = jnewInstance("fr.cnes.celestlab.celestlabx.generic.Params");
  
  // Record parameters in Java object
  for (k = 1 : size(names, 2))
    CLx__setParam(jparams, AllParams(1,Loc(k)), AllParams(2,Loc(k)), params(names(k)));
  end

  jremove(Params); 
  
endfunction








