//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

function [ok] = CLx__execTests(dirpath_tests)
  // Execute all the validation tests of CelestLabX
  //
  // Parameters:
  // - dirpath_tests : (optionnel, string) Tests path. By default = CLx_home()/tests
  // - ok : (boolean) True if all tests succeeded
  //
  // By default, the validation tests are located in one folder:
  // 1) CLx_home()/tests/unit_tests : Contains reference tests
  //    These tests consist of reference inputs, and reference outputs
  //    The test is considered passed if the difference between the reference
  //    output and the output is less than a given treshold (given in absolute
  //    or relative precision).
  //    
  //    These tests are scripts whose file names all end with ".tst"
  //    The only requirement for these scripts is that they must define an array
  //    TEST_OK that contains booleans.
  //    If all the booleans of this array are %T, the test is considered passed.
  //

  // Main function to execute a test
  // fpath: (string) Full file path (1x1)
  // TEST_OK: Array of booleans, %t if test ok (1xN)
  // err: [] if no error, <> [] otherwise 
  function [TEST_OK, err] = executeTest(fpath)
    ier = exec(fpath, 'errcatch', -1);
    err = []; 
    if (ier <> 0)
      err = 1;
      TEST_OK = %f;
    end
  endfunction

  // -----------------------------
  // MAIN FUNCTION  
  // -----------------------------

  // ---------------------------------------------------
  // Execute all tests and print on the screen 
  // and in the log file (all_msg) the results
  // ---------------------------------------------------
  if (~exists("dirpath_tests","local"))
    dirpath_tests = fullfile(CLx_home(), "tests", "unit_tests");
  end
  log_file = fullfile(CLx_home(), "test_log.txt");

  msglog = ["--- Results for all tests "; ..
  msprintf("--- Test directory = %s", dirpath_tests); "" ] ;

  mprintf("%s\n", msglog);

  fpaths = matrix(gsort(fullfile(dirpath_tests, findfiles(dirpath_tests, "*.tst")), "lc", "i"), 1, -1);
  fnames = getrelativefilename(repmat(dirpath_tests, size(fpaths)), fpaths);

  tests_not_OK = [];
  ok = %f; 
  
  for k = 1 : size(fpaths,2)
    fpath = fpaths(k);

    fname = fnames(k);

    mprintf("=> %s\n", fname);

    [TEST_OK, err] = executeTest(fpath);

    msg = ""; 
    
    if (err <> [])
      msg = msprintf("%s: UNEXPECTED ERROR", fname); 
      mprintf("%s\n", msg);
      tests_not_OK = [tests_not_OK; fname]; 
   
    else
      I = find(~TEST_OK); 
      
      if (I == []); 
        msg = msprintf("%s", fname);
        
      else
        msg = msprintf("%s: NOT OK (output #%d)", fname, I(1));
        mprintf("%s\n", msg);
        tests_not_OK = [tests_not_OK; fname]; 

      end
      
    end
    
    msglog = [msglog; msg];

  end

  // ------------
  // Conclusion
  // ------------

  if (tests_not_OK == []); 
    msg = "--- Summary: all tests are OK "; 
    ok = %t; 
  else
    msg = ["--- Summary: one or more test is not OK: "; tests_not_OK]; 
    ok = %f; 
  end

  mprintf("\n");
  mprintf("%s\n", msg);
  msglog = [msglog; ""; msg]; 

  // save messages in log file
  mputl(msglog, log_file);

endfunction

