//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'https://cecill.info'.

#include <nrlmsise-00.h>
#include <clx_msis00.h>
#include <stdio.h>

/* Interface for gtd7d 
   (vectorization)
   NB: the units are USI (kg, m, K) 
   
   Inputs: 
   N: number of values (dimension of all vectors)
   year: year (not used)
   doy: day of year: 1 january 0h - 23h59.99 <=> doy = 1
   sec: seconds of day (UTC) 
   glon, glat, galt: geodetic longitude, latitude, altitude [km and deg] 
   lst: local time for current position and time [hours] (approx: lst = sec/3600 + glon/15)
   f107a: average F10.7 flux (1xN)
   f107: F10.7 flux (1xN) 
   ap: ap indices. (7xN), in the order: 7 ap for N=1, 7 ap for n = 2 ...
   nb_ap: number of ap: 1 or 7 
   
   Outputs: 
   d: densities [kg/m3 or 1/m^3] (9xN) - see gtd7d
   t: temperatures [K] (2xN) - see gtd7d
   
   for k = 0 .. N-1: 
   d[0+9*k] - HE NUMBER DENSITY
   d[1+9*k] - O NUMBER DENSITY
   d[2+9*k] - N2 NUMBER DENSITY
   d[3+9*k] - O2 NUMBER DENSITY
   d[4+9*k] - AR NUMBER DENSITY                       
   d[5+9*k] - TOTAL MASS DENSITY [includes d[8] in td7d]
   d[6+9*k] - H NUMBER DENSITY
   d[7+9*k] - N NUMBER DENSITY
   d[8+9*k] - Anomalous oxygen NUMBER DENSITY


   t[0+2*k] - EXOSPHERIC TEMPERATURE
   t[1+2*k] - TEMPERATURE AT ALT
 
 */ 

void clx_msis00(int N, int *year, int *doy, double *sec, 
                double *glon, double *glat, double *galt, double *lst, 
                double *f107, double *f107A, double *ap, int nb_ap, 
                double *d, double *t)
{
  int i, k; 

  struct nrlmsise_input input; 
  struct nrlmsise_output output; 
  struct nrlmsise_flags flags; 
  struct ap_array apa; 

  // init flags; 
  flags.switches[0] = 1; // USI
  for (i=1; i<24; i++) flags.switches[i] = 1;
  if (nb_ap != 1) flags.switches[9] = -1; // uses 1 or 7 ap values

  // loop on all arrays
  for (i = 0; i < N; i++) { 
    // initialize inputs
    input.doy = doy[i];
    input.year = 0; /* without effect */
    input.sec = sec[i];
    input.g_long = glon[i]; // degrees
    input.g_lat = glat[i]; // degrees
    input.alt = galt[i]; // km
    input.lst = lst[i]; // hours 
    input.f107A = f107A[i];
    input.f107 = f107[i];
    
    // if 1 ap index used
    input.ap = ap[7 * i]; 
    // if 7 ap indices used
    // for (k=0; k<7; k++) apa.a[k] = ap[7 * i + k]; 
    // input.ap_a = &apa;
    input.ap_a = &ap[7 * i]; 
    
    // total density includes anomalous oxygen
    gtd7d(&input, &flags, &output);
   
    // densities 
    for (k=0; k<9; k++) d[9 * i + k] = output.d[k]; 
    
    // temperatures
    for (k=0; k<2; k++) t[2 * i + k] = output.t[k]; 

  }

}






