#!/bin/ksh

####
## STELA batch mode Unix example
## Tested on Linux CentOS 5.5 only
##
## The STELA simulation "example_batch_sim.xml" is loaded and extrapolated,
## then the semi major axis value is increased by one meter before another extrapolation.
## In the end, we get five "output" simulations named "1_" to "5_example_batch_sim.xml".
## The value is modified in the xml file through unix shell commands.
## It is not recommended for novice users to modify METHODS part.
## For more details, see User manual.
####

# STELA_ROOT set during install. Do not change unless you know what you are doing
STELA_ROOT="$INSTALL_PATH"
# Unlike the STELA launcher, the batch launcher can use a pre-set STELA_ROOT
export STELA_ROOT

workfile="/tmp/stela_work_$$_sim.xml"
tmpfile="/tmp/stela_tmp_$$_sim.xml"

######################################## METHODS ########################################

end() {
    CR=$1
    mess=$2
    echo "END PROG (CR="${CR}") # "${mess}
    exit $CR
}

#Getvalue on workfile :: ${workfilename}
# -param1 key in xml file
getValue() {
	key=$1
	if [ $# -ne 1 ]; then
		end 10 "Check usage of function getValue"
	fi

	#Get line in file
	line=$(cat -n "${workfile}"|grep ${key}|awk '{ print $1 }'|head -n1)
	if [ -z "$line" ]; then
		end 20 "Key ${key} not found"
	fi 
	# get value
    value=$(cat "${workfile}"|head -n$line|tail -n1|cut -d'>' -f2|cut -d'<' -f1)
	if [ -z "$value" ]; then
		end 30 "Key ${key} not found"
	fi 

	echo $value
}

#Getvalue of final state on workfile :: ${workfilename}
# -param1 key in xml file
getFinalStateValue() {
	key=$1
	if [ $# -ne 1 ]; then
		end 40 "Check usage of function getFinalStateValue"
	fi

	#Get line in file
	line=$(cat -n "${workfile}"|grep ${key}|awk '{ print $1 }'|head -n2|tail -n1)
	if [ -z "$line" ]; then
		end 50 "Key ${key} not found"
	fi 
	 # get value
    value=$(cat "${workfile}"|head -n$line|tail -n1|cut -d'>' -f2|cut -d'<' -f1)
	if [ -z "$value" ]; then
		end 60 "Key ${key} not found"
	fi 
	echo $value
}

#Set value in workfile :: ${workfilename}
# -param1 key in xml file
# -param2 the new value
setValue() {
	key=$1
	value=$2
	if [ $# -ne 2 ]; then
		end 70 "Check usage of function setValue"
	fi

	#TEMPORARY FILE
	head_tmpfile="/tmp/head_tmp_$$.xml"
	line_tmpfile="/tmp/line_tmp_$$.xml"
	tail_tmpfile="/tmp/tail_tmp_$$.xml"	

	line=$(cat -n "${workfile}"|grep ${key}|awk '{ print $1 }'|head -n1)

	#copy header of file
	cat "${workfile}"|head -n$(( $line - 1 )) >  ${head_tmpfile}

	#curent line of file
	cat "${workfile}"|head -n$(( $line  ))|tail -n1 >  ${line_tmpfile}

	#end of file
    	cat "${workfile}"|tail -n+$(( $line + 1 )) > ${tail_tmpfile}

	#create the new line
	newline="$(cat ${line_tmpfile}|cut -d'>' -f1)"
	newline="${newline}>${value}<$(cat ${line_tmpfile}|cut -d'<' -f3)"
	
	#concate of the new file
	cat ${head_tmpfile} 	> ${workfile}
	echo "${newline}" 	>> ${workfile}
	cat ${tail_tmpfile} 	>> ${workfile}

	#Remove temporary file
	rm ${head_tmpfile} ${tail_tmpfile} ${line_tmpfile}
}

#Extrapolate with the work file :  ${workfilename}
# -no-param
extrapolate(){
	#launch extrapolation
	${STELA_ROOT}/bin/stela-batch.sh -i ${workfile} -o "${tmpfile}"
	CR=$?
	if [ $CR -ne 0 ]; then
		end $CR "Error during extrapolation"
	fi

	if [ ! -f  ${tmpfile} ]; then
		end 80 "Error during extrapolation"
	fi
	
	#move outfile in workfile
	mv ${tmpfile} ${workfile}
	if [ $? -ne 0 ]; then
		end 90 "Error during copy temporary file"
	fi
}

#Send message in a logfile, they are automatically separeted by ';'
# -param1 filename of logfile
# -param* value 
logfile(){
	logfilename=$1
	shift
	separator=';'
	tmp=''
	
	for mess in $*; do
	        if [ -z "$tmp"  ]; then
        		tmp=${mess}
        	else
            		tmp=${tmp}${separator}${mess}
		fi
	done
	
	echo $tmp >> $logfilename
}

#Send header message in a logfile, they are automatically separated by a space character
# -param1 filename of logfile
# -param* header information
 
logfileheader(){
	logfilename=$1
	shift
	separator=';'
	tmp=''
	for mess in $*; do
            	tmp="${tmp} ${mess}"
	done
	
	echo $tmp >> $logfilename
}

######################################## MAIN ########################################

# Input file
filename="example_LEO_sim.xml"
# Log file
logfile="$HOME/stela_batch-log_file.csv"
 
if [ ! -f $filename ]; then
    echo "$filename not found!"
    end 100 "File ${filename} not found"
fi

# Input filename saved
cp $filename ${workfile}

#Initialize logfile 
>${logfile}
logfileheader ${logfile} "#STELA 3.2"
logfileheader ${logfile} "# By <Your name>"
logfileheader ${logfile} "# " "$(date -R)"

logfile ${logfile} 'semiMajorAxis(BULL_IN)' 'eccentricity(BULL_IN)' 'inclination(BULL_IN)' 'rAAN(BULL_IN)' 'argOfPerigee(BULL_IN)' 'meanAnomaly(BULL_IN)' \
			'semiMajorAxis(BULL_OUT)' 'eccentricity(BULL_OUT)' 'inclination(BULL_OUT)' 'rAAN(BULL_OUT)' 'argOfPerigee(BULL_OUT)' 'meanAnomaly(BULL_OUT)'

# Number of runs
nbRuns="10"

echo "=========== START ==========="

for i in $(seq 1 $nbRuns); do
    echo "=========== LOOP " ${i} " ==========="
    
    #setValue "simulationDuration" "1"
    setValue "mass" "1"
    #Launch extrapolation
    extrapolate

    #Log information in a file
    logfile ${logfile} $(getValue "semiMajorAxis") $(getValue "eccentricity") $(getValue "inclination") $(getValue "rAAN") $(getValue "argOfPerigee") $(getValue "meanAnomaly")\
                       $(getFinalStateValue "semiMajorAxis") $(getFinalStateValue "eccentricity") $(getFinalStateValue "inclination") $(getFinalStateValue "rAAN") $(getFinalStateValue "argOfPerigee")  $(getFinalStateValue "meanAnomaly")

    echo "initial value of semiMajorAxis : $( getValue "semiMajorAxis") m"
    echo "final value of semiMajorAxis   : $( getFinalStateValue "semiMajorAxis") m"

    #change value
    value=$( getValue "semiMajorAxis")
    newvalue=$( echo "$value - 1000"|bc -l )
    setValue "semiMajorAxis" "$newvalue"
done

echo "=========== END ==========="
echo "Results are available in file : ${logfile}"

######################################## MAIN ########################################
#clean rep
rm ${workfile} ${tmpfile} 2>/dev/null
end 0 "End prog"
